<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Subscription Cancelled</title>
</head>
<body style="margin: 0; padding: 0; font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, 'Helvetica Neue', Arial, sans-serif; background-color: #0F0F0F;">
    <table role="presentation" style="width: 100%; border-collapse: collapse; background-color: #0F0F0F;">
        <tr>
            <td align="center" style="padding: 40px 20px;">
                <table role="presentation" style="max-width: 600px; width: 100%; border-collapse: collapse; background-color: #1A1A1A; border-radius: 12px; overflow: hidden; box-shadow: 0 4px 20px rgba(0, 0, 0, 0.5);">
                    <!-- Header -->
                    <tr>
                        <td style="padding: 40px 40px 20px; text-align: center; border-bottom: 3px solid #8B5CF6;">
                            <h1 style="margin: 0; font-size: 32px; font-weight: 700; color: #8B5CF6;">
                                🧠 Remembra
                            </h1>
                        </td>
                    </tr>

                    <!-- Body -->
                    <tr>
                        <td style="padding: 40px;">
                            <h2 style="margin: 0 0 20px; font-size: 26px; font-weight: 700; color: #FFFFFF;">
                                Subscription Cancelled
                            </h2>
                            
                            <p style="margin: 0 0 20px; font-size: 16px; line-height: 1.8; color: #D0D0D0;">
                                We're sorry to see you go! Your <strong style="color: #FFFFFF;">{plan}</strong> subscription has been cancelled.
                            </p>

                            <!-- Cancellation Details -->
                            <div style="background-color: #252525; border-left: 4px solid #8B5CF6; padding: 20px; margin: 30px 0; border-radius: 4px;">
                                <p style="margin: 0 0 10px; font-size: 15px; color: #D0D0D0;">
                                    <strong style="color: #FFFFFF;">Access until:</strong> {cancel_date}
                                </p>
                                <p style="margin: 0; font-size: 15px; color: #D0D0D0;">
                                    You'll still have full access to your Pro features until this date.
                                </p>
                            </div>

                            <h3 style="margin: 30px 0 15px; font-size: 20px; font-weight: 600; color: #FFFFFF;">
                                What Happens Next?
                            </h3>

                            <div style="margin: 20px 0;">
                                <!-- Step 1 -->
                                <div style="background-color: #252525; border-radius: 8px; padding: 20px; margin-bottom: 15px;">
                                    <p style="margin: 0 0 8px; font-size: 16px; font-weight: 600; color: #FFFFFF;">
                                        📅 Until {cancel_date}
                                    </p>
                                    <p style="margin: 0; font-size: 15px; color: #D0D0D0; line-height: 1.6;">
                                        Continue using all {plan} features with no restrictions.
                                    </p>
                                </div>

                                <!-- Step 2 -->
                                <div style="background-color: #252525; border-radius: 8px; padding: 20px; margin-bottom: 15px;">
                                    <p style="margin: 0 0 8px; font-size: 16px; font-weight: 600; color: #FFFFFF;">
                                        🔄 After {cancel_date}
                                    </p>
                                    <p style="margin: 0; font-size: 15px; color: #D0D0D0; line-height: 1.6;">
                                        Your account will be downgraded to the Free plan with 1,000 memories.
                                    </p>
                                </div>

                                <!-- Step 3 -->
                                <div style="background-color: #252525; border-radius: 8px; padding: 20px;">
                                    <p style="margin: 0 0 8px; font-size: 16px; font-weight: 600; color: #FFFFFF;">
                                        💾 Your Data is Safe
                                    </p>
                                    <p style="margin: 0; font-size: 15px; color: #D0D0D0; line-height: 1.6;">
                                        All your memories will be preserved. You can export or delete them anytime.
                                    </p>
                                </div>
                            </div>

                            <!-- Feedback Box -->
                            <div style="background: linear-gradient(135deg, #E0E7FF 0%, #C7D2FE 100%); border-left: 4px solid #6366F1; padding: 20px; margin: 30px 0; border-radius: 8px;">
                                <p style="margin: 0 0 10px; font-size: 16px; font-weight: 600; color: #312E81;">
                                    We'd love your feedback
                                </p>
                                <p style="margin: 0; font-size: 15px; color: #4338CA; line-height: 1.6;">
                                    What could we have done better? Reply to this email and let us know how we can improve.
                                </p>
                            </div>

                            <!-- Change your mind? -->
                            <div style="text-align: center; background-color: #252525; border-radius: 8px; padding: 30px; margin: 30px 0;">
                                <p style="margin: 0 0 20px; font-size: 18px; font-weight: 600; color: #FFFFFF;">
                                    Changed your mind?
                                </p>
                                <p style="margin: 0 0 25px; font-size: 15px; color: #D0D0D0;">
                                    You can reactivate your subscription anytime before {cancel_date}.
                                </p>
                                <a href="{resubscribe_url}" style="display: inline-block; background: linear-gradient(135deg, #8B5CF6 0%, #6D28D9 100%); color: #FFFFFF; text-decoration: none; padding: 14px 32px; border-radius: 8px; font-weight: 600; font-size: 15px; box-shadow: 0 4px 15px rgba(139, 92, 246, 0.4);">
                                    Reactivate Subscription →
                                </a>
                            </div>

                            <div style="text-align: center; margin: 20px 0;">
                                <a href="{dashboard_url}" style="color: #8B5CF6; text-decoration: none; font-size: 15px;">
                                    View Dashboard →
                                </a>
                            </div>
                        </td>
                    </tr>

                    <!-- Footer -->
                    <tr>
                        <td style="padding: 30px 40px; background-color: #0F0F0F; border-top: 1px solid #333;">
                            <p style="margin: 0 0 10px; font-size: 14px; color: #666; line-height: 1.6;">
                                Thank you for being a Remembra user. We hope to see you again soon.
                            </p>
                            <p style="margin: 0; font-size: 13px; color: #555;">
                                Remembra &copy; 2026 | 
                                <a href="https://app.remembra.dev/settings" style="color: #666; text-decoration: none;">Preferences</a> | 
                                <a href="https://app.remembra.dev/unsubscribe" style="color: #666; text-decoration: none;">Unsubscribe</a>
                            </p>
                        </td>
                    </tr>
                </table>
            </td>
        </tr>
    </table>
</body>
</html>
